@@ -24,6 +24,7 @@ from .addressableaccount import AddressableAccount
 from .context import Context
 from .encoding import encode_key
 from .group import Group
+from .instrumentvalue import InstrumentValue
 from .layouts import layouts
 from .metadata import Metadata
 from .openorders import OpenOrders
@@ -31,57 +32,44 @@ from .orders import Side
 from .perpaccount import PerpAccount
 from .perpopenorders import PerpOpenOrders
 from .placedorder import PlacedOrder
-from .token import Token
+from .token import Instrument, Token
 from .tokeninfo import TokenInfo
-from .tokenvalue import TokenValue
 from .version import Version
 
 
-# # 🥭 AccountBasketToken class
+# # 🥭 AccountSlot class
 #
-# `AccountBasketToken` gathers basket items together instead of separate arrays.
+# `AccountSlot` gathers slot items together instead of separate arrays.
 #
-class AccountBasketToken:
-    def __init__(self, token_info: TokenInfo, deposit: TokenValue, borrow: TokenValue):
-        self.token_info: TokenInfo = token_info
-        self.deposit: TokenValue = deposit
-        self.borrow: TokenValue = borrow
-
-    @property
-    def net_value(self) -> TokenValue:
-        return self.deposit - self.borrow
-
-    def __str__(self) -> str:
-        return f"""« 𝙰𝚌𝚌𝚘𝚞𝚗𝚝𝙱𝚊𝚜𝚔𝚎𝚝𝚃𝚘𝚔𝚎𝚗 {self.token_info.token.symbol}
-    Net Value:     {self.net_value}
-        Deposited: {self.deposit}
-        Borrowed:  {self.borrow}
-»"""
-
-    def __repr__(self) -> str:
-        return f"{self}"
-
-
-# # 🥭 AccountBasketBaseToken class
-#
-# `AccountBasketBaseToken` is a more specialised `AccountBasketToken` for all the base tokens in the
-# account.
-#
-class AccountBasketBaseToken(AccountBasketToken):
-    def __init__(self, token_info: TokenInfo, quote_token_info: TokenInfo, deposit: TokenValue, borrow: TokenValue, spot_open_orders: typing.Optional[PublicKey], perp_account: typing.Optional[PerpAccount]):
-        super().__init__(token_info, deposit, borrow)
+class AccountSlot:
+    def __init__(self, index: int, base_instrument: Instrument, base_token_info: typing.Optional[TokenInfo], quote_token_info: TokenInfo, raw_deposit: Decimal, deposit: InstrumentValue, raw_borrow: Decimal, borrow: InstrumentValue, spot_open_orders: typing.Optional[PublicKey], perp_account: typing.Optional[PerpAccount]) -> None:
+        self.index: int = index
+        self.base_instrument: Instrument = base_instrument
+        self.base_token_info: typing.Optional[TokenInfo] = base_token_info
         self.quote_token_info: TokenInfo = quote_token_info
+        self.raw_deposit: Decimal = raw_deposit
+        self.deposit: InstrumentValue = deposit
+        self.raw_borrow: Decimal = raw_borrow
+        self.borrow: InstrumentValue = borrow
         self.spot_open_orders: typing.Optional[PublicKey] = spot_open_orders
         self.perp_account: typing.Optional[PerpAccount] = perp_account
 
+    @property
+    def net_value(self) -> InstrumentValue:
+        return self.deposit - self.borrow
+
+    @property
+    def raw_net_value(self) -> Decimal:
+        return self.raw_deposit - self.raw_borrow
+
     def __str__(self) -> str:
         perp_account: str = "None"
         if self.perp_account is not None:
             perp_account = f"{self.perp_account}".replace("\n", "\n        ")
-        return f"""« 𝙰𝚌𝚌𝚘𝚞𝚗𝚝𝙱𝚊𝚜𝚔𝚎𝚝𝙱𝚊𝚜𝚎𝚃𝚘𝚔𝚎𝚗 {self.token_info.token.symbol}
+        return f"""« 𝙰𝚌𝚌𝚘𝚞𝚗𝚝𝚂𝚕𝚘𝚝 {self.base_instrument.symbol}
     Net Value:     {self.net_value}
-        Deposited: {self.deposit}
-        Borrowed:  {self.borrow}
+        Deposited: {self.deposit} (raw value: {self.raw_deposit})
+        Borrowed:  {self.borrow} (raw value {self.raw_borrow})
     Spot OpenOrders: {self.spot_open_orders or "None"}
     Perp Account:
         {perp_account}
@@ -91,9 +79,6 @@ class AccountBasketBaseToken(AccountBasketToken):
         return f"{self}"
 
 
-TMappedAccountBasketValue = typing.TypeVar("TMappedAccountBasketValue")
-
-
 # # 🥭 Account class
 #
 # `Account` holds information about the account for a particular user/wallet for a particualr `Group`.
@@ -101,11 +86,11 @@ TMappedAccountBasketValue = typing.TypeVar("TMappedAccountBasketValue")
 class Account(AddressableAccount):
     def __init__(self, account_info: AccountInfo, version: Version,
                  meta_data: Metadata, group_name: str, group_address: PublicKey, owner: PublicKey,
-                 info: str, shared_quote_token: AccountBasketToken,
+                 info: str, shared_quote: AccountSlot,
                  in_margin_basket: typing.Sequence[bool],
-                 basket_indices: typing.Sequence[bool],
-                 basket: typing.Sequence[AccountBasketBaseToken],
-                 msrm_amount: Decimal, being_liquidated: bool, is_bankrupt: bool):
+                 slot_indices: typing.Sequence[bool],
+                 slots: typing.Sequence[AccountSlot],
+                 msrm_amount: Decimal, being_liquidated: bool, is_bankrupt: bool) -> None:
         super().__init__(account_info)
         self.version: Version = version
 
@@ -114,24 +99,86 @@ class Account(AddressableAccount):
         self.group_address: PublicKey = group_address
         self.owner: PublicKey = owner
         self.info: str = info
-        self.shared_quote_token: AccountBasketToken = shared_quote_token
+        self.shared_quote: AccountSlot = shared_quote
         self.in_margin_basket: typing.Sequence[bool] = in_margin_basket
-        self.basket_indices: typing.Sequence[bool] = basket_indices
-        self.basket: typing.Sequence[AccountBasketBaseToken] = basket
+        self.slot_indices: typing.Sequence[bool] = slot_indices
+        self.slots: typing.Sequence[AccountSlot] = slots
         self.msrm_amount: Decimal = msrm_amount
         self.being_liquidated: bool = being_liquidated
         self.is_bankrupt: bool = is_bankrupt
 
+    @property
+    def shared_quote_token(self) -> Token:
+        token_info = self.shared_quote.base_token_info
+        if token_info is None:
+            raise Exception(f"Shared quote does not have a token: {self.shared_quote}")
+        return Token.ensure(token_info.token)
+
+    @property
+    def slots_by_index(self) -> typing.Sequence[typing.Optional[AccountSlot]]:
+        mapped_items: typing.List[typing.Optional[AccountSlot]] = []
+        slot_counter = 0
+        for available in self.slot_indices:
+            if available:
+                mapped_items += [self.slots[slot_counter]]
+                slot_counter += 1
+            else:
+                mapped_items += [None]
+        mapped_items += [self.shared_quote]
+
+        return mapped_items
+
+    @property
+    def deposits(self) -> typing.Sequence[InstrumentValue]:
+        return [slot.deposit for slot in self.slots]
+
+    @property
+    def deposits_by_index(self) -> typing.Sequence[typing.Optional[InstrumentValue]]:
+        return [slot.deposit if slot is not None else None for slot in self.slots_by_index]
+
+    @property
+    def borrows(self) -> typing.Sequence[InstrumentValue]:
+        return [slot.borrow for slot in self.slots]
+
+    @property
+    def borrows_by_index(self) -> typing.Sequence[typing.Optional[InstrumentValue]]:
+        return [slot.borrow if slot is not None else None for slot in self.slots_by_index]
+
+    @property
+    def net_values(self) -> typing.Sequence[InstrumentValue]:
+        return [slot.net_value for slot in self.slots]
+
+    @property
+    def net_values_by_index(self) -> typing.Sequence[typing.Optional[InstrumentValue]]:
+        return [slot.net_value if slot is not None else None for slot in self.slots_by_index]
+
+    @property
+    def spot_open_orders(self) -> typing.Sequence[PublicKey]:
+        return [slot.spot_open_orders for slot in self.slots if slot.spot_open_orders is not None]
+
+    @property
+    def spot_open_orders_by_index(self) -> typing.Sequence[typing.Optional[PublicKey]]:
+        return [slot.spot_open_orders if slot is not None else None for slot in self.slots_by_index]
+
+    @property
+    def perp_accounts(self) -> typing.Sequence[PerpAccount]:
+        return [slot.perp_account for slot in self.slots if slot.perp_account is not None]
+
+    @property
+    def perp_accounts_by_index(self) -> typing.Sequence[typing.Optional[PerpAccount]]:
+        return [slot.perp_account if slot is not None else None for slot in self.slots_by_index]
+
     @staticmethod
     def from_layout(layout: typing.Any, account_info: AccountInfo, version: Version, group: Group) -> "Account":
         meta_data = Metadata.from_layout(layout.meta_data)
         owner: PublicKey = layout.owner
         info: str = layout.info
-        mngo_token_info = TokenInfo.find_by_symbol(group.tokens, "MNGO")
+        mngo_token = group.liquidity_incentive_token
         in_margin_basket: typing.Sequence[bool] = list([bool(in_basket) for in_basket in layout.in_margin_basket])
         active_in_basket: typing.List[bool] = []
-        basket: typing.List[AccountBasketBaseToken] = []
-        placed_orders_all_markets: typing.List[typing.List[PlacedOrder]] = [[] for _ in range(len(group.tokens) - 1)]
+        slots: typing.List[AccountSlot] = []
+        placed_orders_all_markets: typing.List[typing.List[PlacedOrder]] = [[]
+                                                                            for _ in range(len(group.slot_indices) - 1)]
         for index, order_market in enumerate(layout.order_market):
             if order_market != 0xFF:
                 side = Side.from_value(layout.order_side[index])
@@ -140,48 +187,61 @@ class Account(AddressableAccount):
                 placed_order = PlacedOrder(id, client_id, side)
                 placed_orders_all_markets[int(order_market)] += [placed_order]
 
-        quote_token_info: typing.Optional[TokenInfo] = group.tokens[-1]
-        if quote_token_info is None:
-            raise Exception(f"Could not determine quote token in group {group.address}")
+        quote_token_info: TokenInfo = group.shared_quote
+        quote_token: Token = group.shared_quote_token
+
+        for index in range(len(group.slots_by_index)):
+            group_slot = group.slots_by_index[index]
+            if group_slot is not None:
+                instrument = group_slot.base_instrument
+                token_info = group_slot.base_token_info
+                raw_deposit: Decimal = Decimal(0)
+                intrinsic_deposit: Decimal = Decimal(0)
+                raw_borrow: Decimal = Decimal(0)
+                intrinsic_borrow: Decimal = Decimal(0)
+                if token_info is not None:
+                    raw_deposit = layout.deposits[index]
+                    intrinsic_deposit = token_info.root_bank.deposit_index * raw_deposit
+                    raw_borrow = layout.borrows[index]
+                    intrinsic_borrow = token_info.root_bank.borrow_index * raw_borrow
+
+                deposit = InstrumentValue(instrument, instrument.shift_to_decimals(intrinsic_deposit))
+                borrow = InstrumentValue(instrument, instrument.shift_to_decimals(intrinsic_borrow))
 
-        for index, token_info in enumerate(group.tokens[:-1]):
-            if token_info:
-                intrinsic_deposit = token_info.root_bank.deposit_index * layout.deposits[index]
-                deposit = TokenValue(token_info.token, token_info.token.shift_to_decimals(intrinsic_deposit))
-                intrinsic_borrow = token_info.root_bank.borrow_index * layout.borrows[index]
-                borrow = TokenValue(token_info.token, token_info.token.shift_to_decimals(intrinsic_borrow))
                 perp_open_orders = PerpOpenOrders(placed_orders_all_markets[index])
-                group_basket_market = group.markets[index]
-                if group_basket_market is None:
-                    raise Exception(f"Could not find group basket market at index {index}.")
+
                 perp_account = PerpAccount.from_layout(
                     layout.perp_accounts[index],
-                    token_info.token,
-                    quote_token_info.token,
+                    instrument,
+                    quote_token,
                     perp_open_orders,
-                    group_basket_market.perp_lot_size_converter,
-                    mngo_token_info.token)
+                    group_slot.perp_lot_size_converter,
+                    mngo_token)
                 spot_open_orders = layout.spot_open_orders[index]
-                basket_item: AccountBasketBaseToken = AccountBasketBaseToken(
-                    token_info, quote_token_info, deposit, borrow, spot_open_orders, perp_account)
-                basket += [basket_item]
+                account_slot: AccountSlot = AccountSlot(index, instrument, token_info, quote_token_info,
+                                                        raw_deposit, deposit, raw_borrow, borrow,
+                                                        spot_open_orders, perp_account)
+
+                slots += [account_slot]
                 active_in_basket += [True]
             else:
                 active_in_basket += [False]
 
-        intrinsic_quote_deposit = quote_token_info.root_bank.deposit_index * layout.deposits[-1]
-        quote_deposit = TokenValue(quote_token_info.token,
-                                   quote_token_info.token.shift_to_decimals(intrinsic_quote_deposit))
-        intrinsic_quote_borrow = quote_token_info.root_bank.borrow_index * layout.borrows[-1]
-        quote_borrow = TokenValue(quote_token_info.token,
-                                  quote_token_info.token.shift_to_decimals(intrinsic_quote_borrow))
-        quote: AccountBasketToken = AccountBasketToken(quote_token_info, quote_deposit, quote_borrow)
+        raw_quote_deposit: Decimal = layout.deposits[-1]
+        intrinsic_quote_deposit = quote_token_info.root_bank.deposit_index * raw_quote_deposit
+        quote_deposit = InstrumentValue(quote_token, quote_token.shift_to_decimals(intrinsic_quote_deposit))
+        raw_quote_borrow: Decimal = layout.borrows[-1]
+        intrinsic_quote_borrow = quote_token_info.root_bank.borrow_index * raw_quote_borrow
+        quote_borrow = InstrumentValue(quote_token, quote_token.shift_to_decimals(intrinsic_quote_borrow))
+        quote: AccountSlot = AccountSlot(len(layout.deposits) - 1, quote_token_info.token, quote_token_info,
+                                         quote_token_info, raw_quote_deposit, quote_deposit, raw_quote_borrow,
+                                         quote_borrow, None, None)
 
         msrm_amount: Decimal = layout.msrm_amount
         being_liquidated: bool = bool(layout.being_liquidated)
         is_bankrupt: bool = bool(layout.is_bankrupt)
 
-        return Account(account_info, version, meta_data, group.name, group.address, owner, info, quote, in_margin_basket, active_in_basket, basket, msrm_amount, being_liquidated, is_bankrupt)
+        return Account(account_info, version, meta_data, group.name, group.address, owner, info, quote, in_margin_basket, active_in_basket, slots, msrm_amount, being_liquidated, is_bankrupt)
 
     @staticmethod
     def parse(account_info: AccountInfo, group: Group) -> "Account":
@@ -260,86 +320,33 @@ class Account(AddressableAccount):
 
         return accounts[0]
 
-    @staticmethod
-    def _map_sequence_to_basket_indices(items: typing.Sequence[AccountBasketBaseToken], in_basket: typing.Sequence[bool], selector: typing.Callable[[typing.Any], TMappedAccountBasketValue]) -> typing.Sequence[typing.Optional[TMappedAccountBasketValue]]:
-        mapped_items: typing.List[typing.Optional[TMappedAccountBasketValue]] = []
-        basket_counter = 0
-        for available in in_basket:
-            if available:
-                mapped_items += [selector(items[basket_counter])]
-                basket_counter += 1
-            else:
-                mapped_items += [None]
-
-        return mapped_items
-
-    @property
-    def basket_tokens(self) -> typing.Sequence[typing.Optional[AccountBasketToken]]:
-        return [
-            *Account._map_sequence_to_basket_indices(self.basket, self.basket_indices, lambda item: item),
-            self.shared_quote_token
-        ]
-
-    @property
-    def deposits(self) -> typing.Sequence[typing.Optional[TokenValue]]:
-        return list(map(lambda basket_token: basket_token.deposit if basket_token else None, self.basket_tokens))
-
-    @property
-    def borrows(self) -> typing.Sequence[typing.Optional[TokenValue]]:
-        return list(map(lambda basket_token: basket_token.borrow if basket_token else None, self.basket_tokens))
-
-    @property
-    def net_assets(self) -> typing.Sequence[typing.Optional[TokenValue]]:
-        return list(map(lambda basket_token: basket_token.net_value if basket_token else None, self.basket_tokens))
-
-    @property
-    def spot_open_orders(self) -> typing.Sequence[typing.Optional[PublicKey]]:
-        return Account._map_sequence_to_basket_indices(self.basket, self.basket_indices, lambda item: item.spot_open_orders)
-
-    @property
-    def perp_accounts(self) -> typing.Sequence[typing.Optional[PerpAccount]]:
-        return Account._map_sequence_to_basket_indices(self.basket, self.basket_indices, lambda item: item.perp_account)
-
-    def find_basket_token(self, token: Token) -> AccountBasketBaseToken:
-        for bt in self.basket:
-            if bt.token_info.token == token:
-                return bt
-
-        raise Exception(f"Could not find token {token} in basket in group {self.address}")
-
     def load_all_spot_open_orders(self, context: Context) -> typing.Dict[str, OpenOrders]:
-        spot_open_orders_addresses = list(
-            [basket_token.spot_open_orders for basket_token in self.basket if basket_token.spot_open_orders is not None])
-        spot_open_orders_account_infos = AccountInfo.load_multiple(context, spot_open_orders_addresses)
+        spot_open_orders_account_infos = AccountInfo.load_multiple(context, self.spot_open_orders)
         spot_open_orders_account_infos_by_address = {
             str(account_info.address): account_info for account_info in spot_open_orders_account_infos}
         spot_open_orders: typing.Dict[str, OpenOrders] = {}
-        for basket_token in self.basket:
-            if basket_token.spot_open_orders is not None:
-                account_info = spot_open_orders_account_infos_by_address[str(basket_token.spot_open_orders)]
-                oo = OpenOrders.parse(account_info, basket_token.token_info.token.decimals,
-                                      self.shared_quote_token.token_info.decimals)
-                spot_open_orders[str(basket_token.spot_open_orders)] = oo
+        for slot in self.slots:
+            if slot.spot_open_orders is not None:
+                account_info = spot_open_orders_account_infos_by_address[str(slot.spot_open_orders)]
+                oo = OpenOrders.parse(account_info, slot.base_instrument.decimals,
+                                      self.shared_quote.base_instrument.decimals)
+                spot_open_orders[str(slot.spot_open_orders)] = oo
         return spot_open_orders
 
     def update_spot_open_orders_for_market(self, spot_market_index: int, spot_open_orders: PublicKey) -> None:
-        indexable_basket: typing.Sequence[typing.Optional[AccountBasketBaseToken]] = Account._map_sequence_to_basket_indices(
-            self.basket, self.basket_indices, lambda item: item)
-        item_to_update = indexable_basket[spot_market_index]
+        item_to_update = self.slots_by_index[spot_market_index]
         if item_to_update is None:
             raise Exception(f"Could not find AccountBasketItem in Account {self.address} at index {spot_market_index}.")
         item_to_update.spot_open_orders = spot_open_orders
 
     def __str__(self) -> str:
         info = f"'{self.info}'" if self.info else "(𝑢𝑛-𝑛𝑎𝑚𝑒𝑑)"
-        shared_quote_token: str = f"{self.shared_quote_token}".replace("\n", "\n        ")
-        basket_count = len(self.basket)
-        basket = "\n        ".join([f"{item}".replace("\n", "\n        ") for item in self.basket])
+        shared_quote: str = f"{self.shared_quote}".replace("\n", "\n        ")
+        slot_count = len(self.slots)
+        slots = "\n        ".join([f"{item}".replace("\n", "\n        ") for item in self.slots])
 
-        tokens_in_basket: typing.Sequence[typing.Optional[Token]] = Account._map_sequence_to_basket_indices(
-            self.basket, self.basket_indices, lambda item: item.token_info.token)
-        symbols_in_basket = list([tok.symbol for tok in tokens_in_basket if tok is not None])
-        in_margin_basket = ", ".join(symbols_in_basket) or "None"
+        symbols: typing.Sequence[str] = [slot.base_instrument.symbol for slot in self.slots]
+        in_margin_basket = ", ".join(symbols) or "None"
         return f"""« 𝙰𝚌𝚌𝚘𝚞𝚗𝚝 {info}, {self.version} [{self.address}]
     {self.meta_data}
     Owner: {self.owner}
@@ -348,10 +355,10 @@ class Account(AddressableAccount):
     Bankrupt? {self.is_bankrupt}
     Being Liquidated? {self.being_liquidated}
     Shared Quote Token:
-        {shared_quote_token}
+        {shared_quote}
     In Basket: {in_margin_basket}
-    Basket [{basket_count} in basket]:
-        {basket}
+    Basket [{slot_count} in basket]:
+        {slots}
 »"""
 
     def __repr__(self) -> str:
