@@ -21,7 +21,7 @@ from decimal import Decimal
 from pyserum._layouts.instructions import INSTRUCTIONS_LAYOUT as PYSERUM_INSTRUCTIONS_LAYOUT, InstructionType as PySerumInstructionType
 from pyserum.enums import OrderType as PySerumOrderType, Side as PySerumSide
 from pyserum.instructions import settle_funds as pyserum_settle_funds, SettleFundsParams as PySerumSettleFundsParams
-from pyserum.market import Market as PySerumMarket
+from pyserum.market.market import Market as PySerumMarket
 from pyserum.open_orders_account import make_create_account_instruction as pyserum_make_create_account_instruction
 from solana.keypair import Keypair
 from solana.publickey import PublicKey
@@ -81,7 +81,6 @@ def build_create_solana_account_instructions(context: Context, wallet: Wallet, m
 # Prefer `build_create_spl_account_instructions()` over this function. This function should be
 # reserved for cases where you specifically don't want the associated token account.
 #
-
 def build_create_spl_account_instructions(context: Context, wallet: Wallet, token: Token, lamports: int = 0) -> CombinableInstructions:
     create_account_instructions = build_create_solana_account_instructions(
         context, wallet, TOKEN_PROGRAM_ID, ACCOUNT_LEN, lamports)
@@ -140,13 +139,13 @@ def build_create_serum_open_orders_instructions(context: Context, wallet: Wallet
 #
 # Creates a Serum order-placing instruction using V3 of the NewOrder instruction.
 #
-def build_serum_place_order_instructions(context: Context, wallet: Wallet, market: PySerumMarket, source: PublicKey, open_orders_address: PublicKey, order_type: OrderType, side: Side, price: Decimal, quantity: Decimal, client_id: int, fee_discount_address: typing.Optional[PublicKey]) -> CombinableInstructions:
+def build_serum_place_order_instructions(context: Context, wallet: Wallet, market: PySerumMarket, source: PublicKey, open_orders_address: PublicKey, order_type: OrderType, side: Side, price: Decimal, quantity: Decimal, client_id: int, fee_discount_address: PublicKey) -> CombinableInstructions:
     serum_order_type: PySerumOrderType = PySerumOrderType.POST_ONLY if order_type == OrderType.POST_ONLY else PySerumOrderType.IOC if order_type == OrderType.IOC else PySerumOrderType.LIMIT
     serum_side: PySerumSide = PySerumSide.SELL if side == Side.SELL else PySerumSide.BUY
 
     instruction = market.make_place_order_instruction(
         source,
-        wallet.to_deprecated_solana_account(),
+        wallet.keypair,
         serum_order_type,
         serum_side,
         float(price),
@@ -163,7 +162,6 @@ def build_serum_place_order_instructions(context: Context, wallet: Wallet, marke
 #
 # Creates an event-consuming 'crank' instruction.
 #
-
 def build_serum_consume_events_instructions(context: Context, market_address: PublicKey, event_queue_address: PublicKey, open_orders_addresses: typing.Sequence[PublicKey], limit: int = 32) -> CombinableInstructions:
     instruction = TransactionInstruction(
         keys=[
@@ -188,7 +186,6 @@ def build_serum_consume_events_instructions(context: Context, market_address: Pu
 #
 # Creates a 'settle' instruction.
 #
-
 def build_serum_settle_instructions(context: Context, wallet: Wallet, market: PySerumMarket, open_orders_address: PublicKey, base_token_account_address: PublicKey, quote_token_account_address: PublicKey) -> CombinableInstructions:
     vault_signer = PublicKey.create_program_address(
         [bytes(market.state.public_key()), market.state.vault_signer_nonce().to_bytes(8, byteorder="little")],
@@ -210,6 +207,7 @@ def build_serum_settle_instructions(context: Context, wallet: Wallet, market: Py
 
     return CombinableInstructions(signers=[], instructions=[instruction])
 
+
 # # 🥭 build_spot_settle_instructions function
 #
 # Creates a 'settle' instruction for spot markets.
@@ -236,8 +234,7 @@ def build_serum_settle_instructions(context: Context, wallet: Wallet, market: Py
 # /// 15. `[writable]` quote_vault_ai - MangoGroup quote vault acc
 # /// 16. `[]` dex_signer_ai - dex PySerumMarket signer account
 # /// 17. `[]` spl token program
-
-
+#
 def build_spot_settle_instructions(context: Context, wallet: Wallet, account: Account,
                                    market: PySerumMarket, group: Group, open_orders_address: PublicKey,
                                    base_rootbank: RootBank, base_nodebank: NodeBank,
@@ -294,8 +291,7 @@ def build_spot_settle_instructions(context: Context, wallet: Wallet, account: Ac
 # immediately if the order is filled (either because it's IOC or because it matches an order on the
 # orderbook).
 #
-
-def build_compound_serum_place_order_instructions(context: Context, wallet: Wallet, market: PySerumMarket, source: PublicKey, open_orders_address: PublicKey, all_open_orders_addresses: typing.Sequence[PublicKey], order_type: OrderType, side: Side, price: Decimal, quantity: Decimal, client_id: int, base_token_account_address: PublicKey, quote_token_account_address: PublicKey, fee_discount_address: typing.Optional[PublicKey], consume_limit: int = 32) -> CombinableInstructions:
+def build_compound_serum_place_order_instructions(context: Context, wallet: Wallet, market: PySerumMarket, source: PublicKey, open_orders_address: PublicKey, all_open_orders_addresses: typing.Sequence[PublicKey], order_type: OrderType, side: Side, price: Decimal, quantity: Decimal, client_id: int, base_token_account_address: PublicKey, quote_token_account_address: PublicKey, fee_discount_address: PublicKey, consume_limit: int = 32) -> CombinableInstructions:
     place_order = build_serum_place_order_instructions(
         context, wallet, market, source, open_orders_address, order_type, side, price, quantity, client_id, fee_discount_address)
     consume_events = build_serum_consume_events_instructions(
@@ -310,8 +306,6 @@ def build_compound_serum_place_order_instructions(context: Context, wallet: Wall
 #
 # Builds the instructions necessary for cancelling a perp order.
 #
-
-
 def build_cancel_perp_order_instructions(context: Context, wallet: Wallet, account: Account, perp_market_details: PerpMarketDetails, order: Order, invalid_id_ok: bool) -> CombinableInstructions:
     # Prefer cancelling by client ID so we don't have to keep track of the order side.
     if order.client_id is not None and order.client_id != 0:
@@ -357,7 +351,7 @@ def build_place_perp_order_instructions(context: Context, wallet: Wallet, group:
     raw_side: int = 1 if side == Side.SELL else 0
     raw_order_type: int = order_type.to_perp()
 
-    base_decimals = perp_market_details.base_token.decimals
+    base_decimals = perp_market_details.base_instrument.decimals
     quote_decimals = perp_market_details.quote_token.decimals
 
     base_factor = Decimal(10) ** base_decimals
@@ -389,7 +383,7 @@ def build_place_perp_order_instructions(context: Context, wallet: Wallet, group:
                 AccountMeta(is_signer=False, is_writable=True, pubkey=perp_market_details.asks),
                 AccountMeta(is_signer=False, is_writable=True, pubkey=perp_market_details.event_queue),
                 *list([AccountMeta(is_signer=False, is_writable=False,
-                                   pubkey=oo_address or SYSTEM_PROGRAM_ADDRESS) for oo_address in account.spot_open_orders])
+                                   pubkey=oo_address or SYSTEM_PROGRAM_ADDRESS) for oo_address in account.spot_open_orders_by_index])
             ],
             program_id=context.mango_program_address,
             data=layouts.PLACE_PERP_ORDER.build(
@@ -517,8 +511,6 @@ def build_deposit_instructions(context: Context, wallet: Wallet, group: Group, a
 #     quantity: u64,
 #     allow_borrow: bool,
 # },
-
-
 def build_withdraw_instructions(context: Context, wallet: Wallet, group: Group, account: Account, root_bank: RootBank, node_bank: NodeBank, token_account: TokenAccount, allow_borrow: bool) -> CombinableInstructions:
     value = token_account.value.shift_to_native().value
     withdraw = TransactionInstruction(
@@ -534,7 +526,7 @@ def build_withdraw_instructions(context: Context, wallet: Wallet, group: Group,
             AccountMeta(is_signer=False, is_writable=False, pubkey=group.signer_key),
             AccountMeta(is_signer=False, is_writable=False, pubkey=TOKEN_PROGRAM_ID),
             *list([AccountMeta(is_signer=False, is_writable=False,
-                               pubkey=oo_address or SYSTEM_PROGRAM_ADDRESS) for oo_address in account.spot_open_orders])
+                               pubkey=oo_address or SYSTEM_PROGRAM_ADDRESS) for oo_address in account.spot_open_orders_by_index])
         ],
         program_id=context.mango_program_address,
         data=layouts.WITHDRAW.build({
@@ -613,9 +605,10 @@ def build_spot_place_order_instructions(context: Context, wallet: Wallet, group:
     instructions: CombinableInstructions = CombinableInstructions.empty()
 
     spot_market_address = market.state.public_key()
-    market_index = group.find_spot_market_index(spot_market_address)
+    slot = group.slot_by_spot_market_address(spot_market_address)
+    market_index = slot.index
 
-    open_orders_address = account.spot_open_orders[market_index]
+    open_orders_address = account.spot_open_orders_by_index[market_index]
     if open_orders_address is None:
         create_open_orders = build_spot_openorders_instructions(context, wallet, group, account, market)
         instructions += create_open_orders
@@ -634,12 +627,12 @@ def build_spot_place_order_instructions(context: Context, wallet: Wallet, group:
         float(quantity)) * market.state.quote_lot_size() * market.state.price_number_to_lots(float(price))
 
     base_token_infos = [
-        token_info for token_info in group.base_tokens if token_info is not None and token_info.token.mint == market.state.base_mint()]
+        token_info for token_info in group.base_tokens_by_index if token_info is not None and isinstance(token_info.token, Token) and token_info.token.mint == market.state.base_mint()]
     if len(base_token_infos) != 1:
         raise Exception(
             f"Could not find base token info for group {group.address} - length was {len(base_token_infos)} when it should be 1.")
     base_token_info = base_token_infos[0]
-    quote_token_info = group.shared_quote_token
+    quote_token_info = group.shared_quote
 
     base_root_bank: RootBank = base_token_info.root_bank
     base_node_bank: NodeBank = base_root_bank.pick_node_bank(context)
@@ -676,7 +669,7 @@ def build_spot_place_order_instructions(context: Context, wallet: Wallet, group:
             AccountMeta(is_signer=False, is_writable=False, pubkey=vault_signer),
             AccountMeta(is_signer=False, is_writable=False, pubkey=fee_discount_address),
             *list([AccountMeta(is_signer=False, is_writable=(oo_address == open_orders_address),
-                               pubkey=oo_address) for oo_address in account.spot_open_orders if oo_address is not None])
+                               pubkey=oo_address) for oo_address in account.spot_open_orders])
         ],
         program_id=context.mango_program_address,
         data=layouts.PLACE_SPOT_ORDER_2.build(
@@ -700,8 +693,6 @@ def build_spot_place_order_instructions(context: Context, wallet: Wallet, group:
 #
 # Builds the instructions necessary for cancelling a spot order.
 #
-
-
 def build_cancel_spot_order_instructions(context: Context, wallet: Wallet, group: Group, account: Account, market: PySerumMarket, order: Order, open_orders_address: PublicKey) -> CombinableInstructions:
     # { buy: 0, sell: 1 }
     raw_side: int = 1 if order.side == Side.SELL else 0
