@@ -33,7 +33,6 @@ from .version import Version
 #
 # Does the orderbook side represent bids or asks?
 #
-
 class OrderBookSideType(enum.Enum):
     # We use strings here so that argparse can work with these as parameters.
     BIDS = "BIDS"
@@ -54,7 +53,7 @@ class PerpOrderBookSide(AddressableAccount):
     def __init__(self, account_info: AccountInfo, version: Version,
                  meta_data: Metadata, perp_market_details: PerpMarketDetails, bump_index: Decimal,
                  free_list_len: Decimal, free_list_head: Decimal, root_node: Decimal,
-                 leaf_count: Decimal, nodes: typing.Any):
+                 leaf_count: Decimal, nodes: typing.Any) -> None:
         super().__init__(account_info)
         self.version: Version = version
 
@@ -114,13 +113,13 @@ class PerpOrderBookSide(AddressableAccount):
                 price = node.key["price"]
                 quantity = node.quantity
 
-                decimals_differential = self.perp_market_details.base_token.decimals - self.perp_market_details.quote_token.decimals
+                decimals_differential = self.perp_market_details.base_instrument.decimals - self.perp_market_details.quote_token.decimals
                 native_to_ui = Decimal(10) ** decimals_differential
                 quote_lot_size = self.perp_market_details.quote_lot_size
                 base_lot_size = self.perp_market_details.base_lot_size
                 actual_price = price * (quote_lot_size / base_lot_size) * native_to_ui
 
-                base_factor = Decimal(10) ** self.perp_market_details.base_token.decimals
+                base_factor = Decimal(10) ** self.perp_market_details.base_instrument.decimals
                 actual_quantity = (quantity * self.perp_market_details.base_lot_size) / base_factor
 
                 orders += [Order(int(node.key["order_id"]),
